{$I PFDEFINE.INC}
{***************************************************************}
{*                       PFKEYS.PAS                            *}
{*                                                             *}
{*                   Gordon M. Cressman                        *}
{*               Research Triangle Institute                   *}
{*                                                             *}
{*                   CompuServe 75410,2451                     *}
{*               Internet gmc%titan@rti.rti.org                *}
{*                                                             *}
{*                        11-06-1989                           *}
{*                                                             *}
{* This source code is hereby released into the Public domain  *}
{* by the author.                                              *}
{***************************************************************}

unit PFKeys;

interface
uses
  {$IFDEF UseMouse}
  TPMouse,
  {$ENDIF}
  TPCrt;


const
  NumFKeys = 6;
  FKeyAtt : Word = $70;
  FKeyRow : Word = 25;
  FKeyButton : ButtonStatus = LeftButton;

type
  FKeyLabel = String[30];
  FKeyLabelArray = array[1..NumFKeys] of FKeyLabel;
  FKeyKeyArray   = array[1..NumFKeys] of Word;

  FKeyInfoRecd =
    record
      FKNum   : Word;
      FKLabel : FKeyLabelArray; {Array of function key labels}
      FKey    : FKeyKeyArray;   {Array of keyboard scancodes}
    end;

  FKeyInfoPtr = ^FKeyInfoRecd;

procedure DispFKeys(KeyInfo : FKeyInfoPtr);
  {-Displays a help message on the screen}

procedure PushFKeys(KeyInfo : FKeyInfoPtr);
  {-Pushes a message on the message stack, then displays it on the screen}

procedure PopFKeys;
  {-Displays the previous help message on the Stack}

implementation

type
  FKeyPtr = ^FKeyRecd;
  FKeyRecd =
    record
      FKeyInfo : FKeyInfoPtr;
      Next : FKeyPtr;
    end;

const
  TopFKey : FKeyPtr = nil;

  procedure DispFKeys(KeyInfo : FKeyInfoPtr);
    {-Displays function key labels on the screen and sets up mouse hot boxes}
  var
    FKeyCol : Word;
    FKeyOfs : Word;
    i : Integer;
  begin
    {$IFDEF UseMouse}
    HideMouse;
    {$ENDIF}
    KeyInfo^.FKNum := 0;
    FKeyCol := 1;
    FKeyOfs := ScreenWidth div NumFKeys;
    FastFill(ScreenWidth, ' ', FKeyRow, 1, FKeyAtt);
    for i := 1 to NumFKeys do
      with KeyInfo^ do
        begin
          if Ord(FKLabel[i][0]) > 0 then
            begin
              FastText(FKLabel[i], FKeyRow, FKeyCol);
              {$IFDEF EnableMouseHotBoxes}
              if MouseInstalled then
                MouseHotBox(FKeyCol, FKeyRow, FKeyCol + Length(FKLabel[i]) - 1,
                            FKeyRow, FKeyButton, FKey[i]);
              {$ENDIF}
              Inc(FKNum);
            end; {if}
          FKeyCol := FKeyCol + FKeyOfs;
        end; {with}
    {$IFDEF UseMouse}
    ShowMouse;
    {$ENDIF}
  end; {DispFKeys}

  procedure PushFKeys(KeyInfo : FKeyInfoPtr);
    {-Pushes new function key settings on the stack, then displays the}
    {function key labels and sets up mouse hot boxes}
  var
    Temp : FKeyPtr;
  begin
    if (SizeOf(FKeyRecd) <= MaxAvail) then
      begin
        GetMem(Temp, SizeOf(FKeyRecd));
        Temp^.FKeyInfo := KeyInfo;
        Temp^.Next := TopFKey;
        TopFKey := Temp;
        DispFKeys(TopFKey^.FKeyInfo);
      end;
  end; {PushFKeys}

  procedure PopFKeys;
    {-Releases the current function key information and mouse hot boxes,}
    {then restores the previous function keys labels and hot boxes.}
  var
    Temp : FKeyPtr;
    i : Integer;
  begin
    if TopFKey^.Next <> nil then
      begin
        Temp := TopFKey;
        {$IFDEF EnableMouseHotBoxes}
        if MouseInstalled then
          for i := 1 to Temp^.FKeyInfo^.FKNum do
            PopMouseHotBox;
        {$ENDIF}
        TopFKey := Temp^.Next;
        FreeMem(Temp, SizeOf(FKeyRecd));
        DispFKeys(TopFKey^.FKeyInfo);
      end;
  end; {PopFKeys}

begin
  {no initialization}
end.{PFkeys}
